@@ -1,6 +1,6 @@
 package HTML::Parser;
 
-# Copyright 1996-2001, Gisle Aas.
+# Copyright 1996-2004, Gisle Aas.
 # Copyright 1999-2000, Michael A. Chase.
 #
 # This library is free software; you can redistribute it and/or
@@ -9,7 +9,7 @@ package HTML::Parser;
 use strict;
 use vars qw($VERSION @ISA);
 
-$VERSION = '3.1994';  # $Date: 2001/03/30 08:15:18 $
+$VERSION = '3.3990';  # $Date: 2004/11/17 14:29:28 $
 
 require HTML::Entities;
 
@@ -159,31 +159,6 @@ HTML::Parser - HTML parser class
  open(F, "foo.html") || die;
  $p->parse_file(*F);
 
-HTML::Parser version 2 style subclassing and method callbacks:
-
- {
-    package MyParser;
-    use base 'HTML::Parser';
-
-    sub start {
-       my($self, $tagname, $attr, $attrseq, $origtext) = @_;
-       #...
-    }
-
-    sub end {
-	my($self, $tagname, $origtext) = @_;
-	#...
-    }
-
-    sub text {
-	my($self, $origtext, $is_cdata) = @_;
-	#...
-    }
- }
-
- my $p = MyParser->new;
- $p->parse_file("foo.html");
-
 =head1 DESCRIPTION
 
 Objects of the C<HTML::Parser> class will recognize markup and
@@ -191,11 +166,11 @@ separate it from plain text (alias data content) in HTML
 documents.  As different kinds of markup and text are recognized, the
 corresponding event handlers are invoked.
 
-C<HTML::Parser> in not a generic SGML parser.  We have tried to
+C<HTML::Parser> is not a generic SGML parser.  We have tried to
 make it able to deal with the HTML that is actually "out there", and
 it normally parses as closely as possible to the way the popular web
 browsers do it instead of strictly following one of the many HTML
-specifications from W3C.  Where there is disagreement there is often
+specifications from W3C.  Where there is disagreement, there is often
 an option that you can enable to get the official behaviour.
 
 The document to be parsed may be supplied in arbitrary chunks.  This
@@ -203,9 +178,8 @@ makes on-the-fly parsing as documents are received from the network
 possible.
 
 If event driven parsing does not feel right for your application, you
-might want to use C<HTML::PullParser>.  It is a
-C<HTML::Parser> subclass that allows a more conventional program
-structure.
+might want to use C<HTML::PullParser>.  This is an C<HTML::Parser>
+subclass that allows a more conventional program structure.
 
 
 =head1 METHODS
@@ -217,11 +191,11 @@ The following method is used to construct a new C<HTML::Parser> object:
 =item $p = HTML::Parser->new( %options_and_handlers )
 
 This class method creates a new C<HTML::Parser> object and
-returns it.  Key/value pair arguments may be provided to assign event
+returns it.  Key/value argument pairs may be provided to assign event
 handlers or initialize parser options.  The handlers and parser
-options can also be set or modified later by method calls described below.
+options can also be set or modified later by the method calls described below.
 
-If a top level key is in the form "<event>_h" (e.g., "text_h"} then it
+If a top level key is in the form "<event>_h" (e.g., "text_h") then it
 assigns a handler to that event, otherwise it initializes a parser
 option. The event handler specification value must be an array
 reference.  Multiple handlers may also be assigned with the 'handlers
@@ -231,7 +205,7 @@ If new() is called without any arguments, it will create a parser that
 uses callback methods compatible with version 2 of C<HTML::Parser>.
 See the section on "version 2 compatibility" below for details.
 
-Special constructor option 'api_version => 2' can be used to
+The special constructor option 'api_version => 2' can be used to
 initialize version 2 callbacks while still setting other options and
 handlers.  The 'api_version => 3' option can be used if you don't want
 to set any options and don't want to fall back to v2 compatible
@@ -270,16 +244,38 @@ to the C<HTML::Parser> object:
 
 Parse $string as the next chunk of the HTML document.  The return
 value is normally a reference to the parser object (i.e. $p).
-Handlers invoked should not attempt modify the $string in-place until
+Handlers invoked should not attempt to modify the $string in-place until
 $p->parse returns.
 
 If an invoked event handler aborts parsing by calling $p->eof, then
 $p->parse() will return a FALSE value.
 
+=item $p->parse( $code_ref )
+
+If a code reference is passed as the argument to be parsed, then the
+chunks to be parsed are obtained by invoking this function repeatedly.
+Parsing continues until the function returns an empty (or undefined)
+result.  When this happens $p->eof is automatically signalled.
+
+Parsing will also abort if one of the event handlers calls $p->eof.
+
+The effect of this is the same as:
+
+ while (1) {
+    my $chunk = &$code_ref();
+    if (!defined($chunk) || !length($chunk)) {
+        $p->eof;
+        return $p;
+    }
+    $p->parse($chunk) || return undef;
+ }
+
+But it is more efficient as this loop runs internally in XS code.
+
 =item $p->parse_file( $file )
 
 Parse text directly from a file.  The $file argument can be a
-filename, an open file handle, or a reference to a an open file
+filename, an open file handle, or a reference to an open file
 handle.
 
 If $file contains a filename and the file can't be opened, then the
@@ -293,9 +289,12 @@ If an invoked event handler aborts parsing by calling $p->eof,
 then $p->parse_file() may not have read the entire file.
 
 On systems with multi-byte line terminators, the values passed for the
-offset and length argspecs may be too low if parse_file is called with
+offset and length argspecs may be too low if parse_file() is called on
 a file handle that is not in binary mode.
 
+If a filename is passed in, then parse_file() will open the file in
+binary mode.
+
 =item $p->eof
 
 Signals the end of the HTML document.  Calling the $p->eof method
@@ -306,7 +305,10 @@ Calling $p->eof inside a handler will terminate parsing at that point
 and cause $p->parse to return a FALSE value.  This also terminates
 parsing by $p->parse_file().
 
-The return value is a reference to the parser object.
+After $p->eof has been called, the parse() and parse_file() methods
+can be invoked to feed new documents with the parser object.
+
+The return value from eof() is a reference to the parser object.
 
 =back
 
@@ -321,10 +323,12 @@ Methods that can be used to get and/or set parser options are:
 
 =over
 
-=item $p->strict_comment( [$bool] )
+=item $p->strict_comment
+
+=item $p->strict_comment( $bool )
 
 By default, comments are terminated by the first occurrence of "-->".
-This is the behaviour of most popular browsers (like Netscape and
+This is the behaviour of most popular browsers (like Mozilla, Opera and
 MSIE), but it is not correct according to the official HTML
 standard.  Officially, you need an even number of "--" tokens before
 the closing ">" is recognized and there may not be anything but
@@ -332,22 +336,41 @@ whitespace between an even and an odd "--".
 
 The official behaviour is enabled by enabling this attribute.
 
-=item $p->strict_names( [$bool] )
+Enabling of 'strict_comment' also disables recognizing these forms as
+comments:
+
+  </ comment>
+  <! comment>
+
+
+=item $p->strict_names
+
+=item $p->strict_names( $bool )
 
 By default, almost anything is allowed in tag and attribute names.
 This is the behaviour of most popular browsers and allows us to parse
-some broken tags with invalid attr values like:
+some broken tags with invalid attribute values like:
 
    <IMG SRC=newprevlstGr.gif ALT=[PREV LIST] BORDER=0>
 
 By default, "LIST]" is parsed as a boolean attribute, not as
 part of the ALT value as was clearly intended.  This is also what
-Netscape sees.
+Mozilla sees.
 
 The official behaviour is enabled by enabling this attribute.  If
 enabled, it will cause the tag above to be reported as text
 since "LIST]" is not a legal attribute name.
 
+=item $p->strict_end
+
+=item $p->strict_end( $bool )
+
+By default, attributes and other junk are allowed to be present on end tags in a
+manner that emulates MSIE's behaviour.
+
+The official behaviour is enabled with this attribute.  If enabled,
+only whitespace is allowed between the tagname and the final ">".
+
 =item $p->boolean_attribute_value( $val )
 
 This method sets the value reported for boolean attributes inside HTML
@@ -355,13 +378,15 @@ start tags.  By default, the name of the attribute is also used as its
 value.  This affects the values reported for C<tokens> and C<attr>
 argspecs.
 
-=item $p->xml_mode( [$bool] )
+=item $p->xml_mode
+
+=item $p->xml_mode( $bool )
 
 Enabling this attribute changes the parser to allow some XML
 constructs such as I<empty element tags> and I<XML processing
 instructions>.  It disables forcing tag and attribute names to lower
 case when they are reported by the C<tagname> and C<attr> argspecs,
-and suppress special treatment of elements that are parsed as CDATA
+and suppresses special treatment of elements that are parsed as CDATA
 for HTML.
 
 I<Empty element tags> look like start tags, but end with the character
@@ -374,12 +399,14 @@ the correct tag name.
 I<XML processing instructions> are terminated by "?>" instead of a
 simple ">" as is the case for HTML.
 
-=item $p->unbroken_text( [$bool] )
+=item $p->unbroken_text
+
+=item $p->unbroken_text( $bool )
 
 By default, blocks of text are given to the text handler as soon as
-possible (but the parser makes sure to always break text at the
+possible (but the parser takes care always to break text at a
 boundary between whitespace and non-whitespace so single words and
-entities always can be decoded safely).  This might create breaks that
+entities can always be decoded safely).  This might create breaks that
 make it hard to do transformations on the text. When this attribute is
 enabled, blocks of text are always reported in one piece.  This will
 delay the text event until the following (non-text) event has been
@@ -390,14 +417,41 @@ segment of text and C<length> is the combined length of the segments.
 Since there might be ignored tags in between, these numbers can't be
 used to directly index in the original document file.
 
-=item $p->marked_sections( [$bool] )
+=item $p->marked_sections
+
+=item $p->marked_sections( $bool )
 
 By default, section markings like <![CDATA[...]]> are treated like
 ordinary text.  When this attribute is enabled section markings are
 honoured.
 
-There are currently no events associated with marked section
-elements.
+There are currently no events associated with the marked section
+markup, but the text can be returned as C<skipped_text>.
+
+=item $p->attr_encoded
+
+=item $p->attr_encoded( $bool )
+
+By default, the C<attr> and C<@attr> argspecs will have general
+entities for attribute values decoded.  Enabling this attribute leaves
+entities alone.
+
+=item $p->case_sensitive
+
+=item $p->case_sensitive( $bool )
+
+By default, tagnames and attribute names are down-cased.  Enabling this
+attribute leaves them as found in the HTML source document.
+
+=item $p->closing_plaintext
+
+=item $p->closing_plaintext( $bool )
+
+By default, "plaintext" element can never be closed. Everything up to
+the end of the document is parsed in CDATA mode.  This historical
+behaviour is what at least MSIE does.  Enabling this attribute makes
+closing "</plaintext>" tag effective and the parsing process will resume
+after seeing this tag.  This emulates gecko-based browsers.
 
 =back
 
@@ -406,11 +460,11 @@ method is used to set up handlers for different events:
 
 =over
 
-=item $p->handler( event => \&subroutine, argspec )
+=item $p->handler( event => \&subroutine, $argspec )
 
-=item $p->handler( event => method_name, argspec )
+=item $p->handler( event => $method_name, $argspec )
 
-=item $p->handler( event => \@accum, argspec )
+=item $p->handler( event => \@accum, $argspec )
 
 =item $p->handler( event => "" );
 
@@ -421,32 +475,33 @@ method is used to set up handlers for different events:
 This method assigns a subroutine, method, or array to handle an event.
 
 Event is one of C<text>, C<start>, C<end>, C<declaration>, C<comment>,
-C<process> or C<default>.
+C<process>, C<start_document>, C<end_document> or C<default>.
 
-I<Subroutine> is a reference to a subroutine which is called to handle
+The C<\&subroutine> is a reference to a subroutine which is called to handle
 the event.
 
-I<Method_name> is the name of a method of $p which is called to handle
+The C<$method_name> is the name of a method of $p which is called to handle
 the event.
 
-I<Accum> is a array that will hold the event information as
+The C<@accum> is an array that will hold the event information as
 sub-arrays.
 
 If the second argument is "", the event is ignored.
 If it is undef, the default handler is invoked for the event.
 
-I<Argspec> is a string that describes the information to be reported
+The C<$argspec> is a string that describes the information to be reported
 for the event.  Any requested information that does not apply to a
 specific event is passed as C<undef>.  If argspec is omitted, then it
-is left unchanged since last update.
+is left unchanged.
 
-The return value from $p->handle is the old callback routine or a
+The return value from $p->handler is the old callback routine or a
 reference to the accumulator array.
 
-Return values from handler callback routines/methods are always
+Any return values from handler callback routines/methods are always
 ignored.  A handler callback can request parsing to be aborted by
 invoking the $p->eof method.  A handler callback is not allowed to
-invoke $p->parse() or $p->parse_file().
+invoke the $p->parse() or $p->parse_file() method.  An exception will
+be raised if it tries.
 
 Examples:
 
@@ -467,49 +522,56 @@ The array elements will be ['S', \%attr, \@attr_seq, $text].
 
    $p->handler(start => "");
 
-This causes 'start' events to be ignored.  It also supresses
-invokations of any default handler for start events.  It is equivalent
-to $p->handler(start => sub {}), but is more efficient.
+This causes 'start' events to be ignored.  It also suppresses
+invocations of any default handler for start events.  It is in most
+cases equivalent to $p->handler(start => sub {}), but is more
+efficient.  It is different from the empty-sub-handler in that
+C<skipped_text> is not reset by it.
 
    $p->handler(start => undef);
 
-This causes no handler to be assosiated with start events.
+This causes no handler to be associated with start events.
 If there is a default handler it will be invoked.
 
 =back
 
 Filters based on tags can be set up to limit the number of events
 reported.  The main bottleneck during parsing is often the huge number
-of callbacks made.  Applying filters can improve performance
-significantly.
+of callbacks made from the parser.  Applying filters can improve
+performance significantly.
 
 The following methods control filters:
 
 =over
 
-=item $p->ignore_tags( TAG, ... )
+=item $p->ignore_tags( @tags )
 
 Any C<start> and C<end> events involving any of the tags given are
 suppressed.
 
-=item $p->report_tags( TAG, ... )
+=item $p->report_tags( @tags )
 
 Any C<start> and C<end> events involving any of the tags I<not> given
 are suppressed.
 
-=item $p->ignore_elements( TAG, ... )
+=item $p->ignore_elements( @tags )
 
-Both the C<start> and the C<end> event as well as any events that
+Both the C<start> event and the C<end> event as well as any events that
 would be reported in between are suppressed.  The ignored elements can
-contain nested occurences of itself.  Example:
+contain nested occurrences of itself.  Example:
 
    $p->ignore_elements(qw(script style));
 
+The C<script> and C<style> tags will always nest properly since their
+content is parsed in CDATA mode.  For most other tags
+C<ignore_elements> must be used with caution since HTML is often not
+I<well formed>.
+
 =back
 
 =head2 Argspec
 
-Argspec is a string containing a comma separated list that describes
+Argspec is a string containing a comma-separated list that describes
 the information reported by the event.  The following argspec
 identifier names can be used:
 
@@ -520,6 +582,12 @@ identifier names can be used:
 Self causes the current object to be passed to the handler.  If the
 handler is a method, this must be the first element in the argspec.
 
+An alternative to passing self as an argspec is to register closures
+that capture $self by themselves as handlers.  Unfortunately this
+creates circular references which prevent the HTML::Parser object
+from being garbage collected.  Using the C<self> argspec avoids this
+problem.
+
 =item C<tokens>
 
 Tokens causes a reference to an array of token strings to be passed.
@@ -533,16 +601,15 @@ For C<comment> events, this contains each sub-comment.  If
 $p->strict_comments is disabled, there will be only one sub-comment.
 
 For C<start> events, this contains the original tag name followed by
-the attribute name/value pairs.  The value of boolean attributes will
-be either the value set by $p->boolean_attribute_value or the
+the attribute name/value pairs.  The values of boolean attributes will
+be either the value set by $p->boolean_attribute_value, or the
 attribute name if no value has been set by
 $p->boolean_attribute_value.
 
-For C<end> events, this contains the original tag name (one token
-only).
+For C<end> events, this contains the original tag name (always one token).
 
-For C<process> events, this contains the process instructions (one
-token only).
+For C<process> events, this contains the process instructions (always one
+token).
 
 This passes C<undef> for C<text> events.
 
@@ -558,7 +625,7 @@ Boolean attributes in a C<start> event will have (0,0) for the
 attribute value offset and length.
 
 This passes undef if there are no tokens in the event (e.g., C<text>)
-and for artifical C<end> events triggered by empty element tags.
+and for artificial C<end> events triggered by empty element tags.
 
 If you are using these offsets and lengths to modify C<text>, you
 should either work from right to left, or be very careful to calculate
@@ -581,11 +648,12 @@ This passes undef if there are no tokens in the event.
 =item C<tagname>
 
 This is the element name (or I<generic identifier> in SGML jargon) for
-start and end tags.  Since HTML is case insensitive this name is
+start and end tags.  Since HTML is case insensitive, this name is
 forced to lower case to ease string matching.
 
-Since XML is case sensitive, the tagname case is not
-changed when C<xml_mode> is enabled.
+Since XML is case sensitive, the tagname case is not changed when
+C<xml_mode> is enabled.  The same happens if the C<case_sensitive> attribute
+is set.
 
 The declaration type of declaration elements is also passed as a tagname,
 even if that is a bit strange.
@@ -604,16 +672,16 @@ Attr causes a reference to a hash of attribute name/value pairs to be
 passed.
 
 Boolean attributes' values are either the value set by
-$p->boolean_attribute_value or the attribute name if no value has been
+$p->boolean_attribute_value, or the attribute name if no value has been
 set by $p->boolean_attribute_value.
 
 This passes undef except for C<start> events.
 
-Unless C<xml_mode> is enabled, the attribute names are forced to
-lower case.
+Unless C<xml_mode> or C<case_sensitive> is enabled, the attribute
+names are forced to lower case.
 
 General entities are decoded in the attribute values and
-one layer of matching quotes enclosing the attribute values are removed.
+one layer of matching quotes enclosing the attribute values is removed.
 
 =item C<attrseq>
 
@@ -623,12 +691,12 @@ the original sequence.
 
 This passes undef except for C<start> events.
 
-Unless C<xml_mode> is enabled, the attribute names are forced to lower
-case.
+Unless C<xml_mode> or C<case_sensitive> is enabled, the attribute
+names are forced to lower case.
 
 =item C<@attr>
 
-Basically same as C<attr>, but keys and values are passed as
+Basically the same as C<attr>, but keys and values are passed as
 individual arguments and the original sequence of the attributes is
 kept.  The parameters passed will be the same as the @attr calculated
 here:
@@ -638,7 +706,7 @@ here:
 assuming $attr and $attrseq here are the hash and array passed as the
 result of C<attr> and C<attrseq> argspecs.
 
-This pass no values for events besides C<start>.
+This passes no values for events besides C<start>.
 
 =item C<text>
 
@@ -649,50 +717,65 @@ passed.
 
 Dtext causes the decoded text to be passed.  General entities are
 automatically decoded unless the event was inside a CDATA section or
-was between literal start and end tags (C<script>, C<style>, C<xmp>,
-and C<plaintext>).
+was between literal start and end tags (C<script>, C<style>,
+C<xmp>, and C<plaintext>).
 
-The ISO 8859-1 character set (aka Latin1) is assumed for entity
-decoding.
-
-It is planned that C<HTML::Parser> will get an C<utf8> option
-at some point that will affect the byte sequence that characters with
-codes greater than 127 will decode into.
+The Unicode character set is assumed for entity decoding.  With Perl
+version < 5.8 only the Latin1 range is supported, and entities for
+characters outside the range 0..255 are left unchanged.
 
 This passes undef except for C<text> events.
 
 =item C<is_cdata>
 
 Is_cdata causes a TRUE value to be passed if the event is inside a CDATA
-section or is between literal start and end tags (C<script>,
+section or between literal start and end tags (C<script>,
 C<style>, C<xmp>, and C<plaintext>).
 
-When the flag is FALSE for a text event, then you should normally
+if the flag is FALSE for a text event, then you should normally
 either use C<dtext> or decode the entities yourself before the text is
 processed further.
 
+=item C<skipped_text>
+
+Skipped_text returns the concatenated text of all the events that have
+been skipped since the last time an event was reported.  Events might
+be skipped because no handler is registered for them or because some
+filter applies.  Skipped text also includes marked section markup,
+since there are no events that can catch it.
+
+If an C<"">-handler is registered for an event, then the text for this
+event is not included in C<skipped_text>.  Skipped text both before
+and after the C<"">-event is included in the next reported
+C<skipped_text>.
+
 =item C<offset>
 
 Offset causes the byte position in the HTML document of the start of
-the event to be passed.  The first byte in the document is 0.
+the event to be passed.  The first byte in the document has offset 0.
 
 =item C<length>
 
 Length causes the number of bytes of the source text of the event to
 be passed.
 
+=item C<offset_end>
+
+Offset_end causes the byte position in the HTML document of the end of
+the event to be passed.  This is the same as C<offset> + C<length>.
+
 =item C<event>
 
 Event causes the event name to be passed.
 
 The event name is one of C<text>, C<start>, C<end>, C<declaration>,
-C<comment>, C<process> or C<default>.
+C<comment>, C<process>, C<start_document> or C<end_document>.
 
 =item C<line>
 
 Line causes the line number of the start of the event to be passed.
 The first line in the document is 1.  Line counting doesn't start
-until at least one handler requests this value.
+until at least one handler requests this value to be reported.
 
 =item C<column>
 
@@ -706,13 +789,23 @@ in single (') or double (") quotes is passed as entered.
 
 =item C<undef>
 
-Pass an undefined value.  Useful as padding.
+Pass an undefined value.  Useful as padding where the same handler
+routine is registered for multiple events.
 
 =back
 
 The whole argspec string can be wrapped up in C<'@{...}'> to signal
-that resulting event array should be flatten.  This only makes a
+that the resulting event array should be flattened.  This only makes a
 difference if an array reference is used as the handler target.
+Consider this example:
+
+   $p->handler(text => [], 'text');
+   $p->handler(text => [], '@{text}']);
+
+With two text events; C<"foo">, C<"bar">; then the first example will end
+up with [["foo"], ["bar"]] and the second with ["foo", "bar"] in
+the handler target array.
+
 
 =head2 Events
 
@@ -722,9 +815,9 @@ Handlers for the following events can be registered:
 
 =item C<text>
 
-This event is triggered when plain text is recognized.  The text may
-contain multiple lines.  A sequence of text may be broken between
-several text events unless $p->unbroken_text is enabled.
+This event is triggered when plain text (characters) is recognized.
+The text may contain multiple lines.  A sequence of text may be broken
+between several text events unless $p->unbroken_text is enabled.
 
 The parser will make sure that it does not break a word or a sequence
 of whitespace between two text events.
@@ -772,7 +865,7 @@ Example:
 This event is triggered when a processing instructions markup is
 recognized.
 
-The format and content of processing instructions is system and
+The format and content of processing instructions are system and
 application dependent.
 
 Examples:
@@ -780,6 +873,17 @@ Examples:
   <? HTML processing instructions >
   <? XML processing instructions ?>
 
+=item C<start_document>
+
+This event is triggered before any other events for a new document.  A
+handler for it can be used to initialize stuff.  There is no document
+text associated with this event.
+
+=item C<end_document>
+
+This event is triggered when $p->eof is called and after any remaining
+text is flushed.  There is no document text associated with this event.
+
 =item C<default>
 
 This event is triggered for events that do not have a specific
@@ -811,7 +915,7 @@ This is equivalent to the following method calls:
 		 $self->declaration(substr($_[0], 2, -1));},
              "self, text");
 
-Setup of these handlers can also be requested with the "api_version =>
+Setting up these handlers can also be requested with the "api_version =>
 2" constructor option.
 
 =head1 SUBCLASSING
@@ -819,7 +923,7 @@ Setup of these handlers can also be requested with the "api_version =>
 The C<HTML::Parser> class is subclassable.  Parser objects are plain
 hashes and C<HTML::Parser> reserves only hash keys that start with
 "_hparser".  The parser state can be set up by invoking the init()
-method which takes the same arguments as new().
+method, which takes the same arguments as new().
 
 =head1 EXAMPLES
 
@@ -832,6 +936,17 @@ does nothing and a default handler that will print out anything else:
                     comment_h => [""],
                    )->parse_file(shift || die) || die $!;
 
+An alternative implementation is:
+
+  use HTML::Parser;
+  HTML::Parser->new(end_document_h => [sub { print shift },
+                                       'skipped_text'],
+                    comment_h      => [""],
+                   )->parse_file(shift || die) || die $!;
+
+This will in most cases be much more efficient since only a single
+callback will be made.
+
 The next example prints out the text that is inside the <title>
 element of an HTML document.  Here we start by setting up a start
 handler.  When it sees the title start tag it enables a text handler
@@ -854,20 +969,26 @@ parsing as soon as the title end tag is seen:
   $p->parse_file(shift || die) || die $!;
   print "\n";
 
-More examples are found in the "eg/" directory of the C<HTML-Parser>
-distribution; the program C<hrefsub> shows how you can edit all links
-found in a document and C<htextsub> how to edid the text only; the
+More examples are found in the F<eg/> directory of the C<HTML-Parser>
+distribution: the program C<hrefsub> shows how you can edit all links
+found in a document; the program C<htextsub> shows how to edit the text only; the
 program C<hstrip> shows how you can strip out certain tags/elements
 and/or attributes; and the program C<htext> show how to obtain the
 plain text, but not any script/style content.
 
+You can browse the F<eg/> directory online from the I<[Browse]> link on
+the http://search.cpan.org/~gaas/HTML-Parser/ page.
+
 =head1 BUGS
 
-C<HTML::Parser> will leave <plaintext> mode when it sees </plaintext>.
-Plaintext mode should not really be escapeable.
+Unicode strings are not parsed correctly.  A workaround is to encode
+them as UTF-8 before passing them to the HTML::Parser.  The C<Encode>
+module can do that.
 
 The <style> and <script> sections do not end with the first "</", but
-need the complete corresponding end tag.
+need the complete corresponding end tag.  MSIE avoids terminating a
+<script> section if the </script> occurs inside quotes.  HTML::Parser
+is not that "smart".
 
 When the I<strict_comment> option is enabled, we still recognize
 comments where there is something other than whitespace between even
@@ -881,9 +1002,9 @@ into the same literal argspec.
 
 Empty tags, e.g. "<>" and "</>", are not recognized.  SGML allows them
 to repeat the previous start tag or close the previous start tag
-respecitvely.
+respectively.
 
-NET tags, e.g. "code/.../" are not recognized.  This is an SGML
+NET tags, e.g. "code/.../" are not recognized.  This is SGML
 shorthand for "<code>...</code>".
 
 Unclosed start or end tags, e.g. "<tt<b>...</b</tt>" are not
@@ -934,7 +1055,7 @@ This is not permitted.
 =item marked sections not supported
 
 (F) The $p->marked_sections() method was invoked in a HTML::Parser
-module that was compiled without support for marked sections. 
+module that was compiled without support for marked sections.
 
 =item Unknown boolean attribute (%d)
 
@@ -996,7 +1117,7 @@ be found at C<http://www.sgml.u-net.com/book/sgml-8.htm>.
 
 =head1 COPYRIGHT
 
- Copyright 1996-2001 Gisle Aas. All rights reserved.
+ Copyright 1996-2004 Gisle Aas. All rights reserved.
  Copyright 1999-2000 Michael A. Chase.  All rights reserved.
 
 This library is free software; you can redistribute it and/or
