@@ -1,6 +1,6 @@
-/* $Id: hparser.c,v 2.98 2004/11/11 10:12:51 gisle Exp $
+/* $Id: hparser.c,v 2.104 2004/11/17 14:09:41 gisle Exp $
  *
- * Copyright 1999-2002, Gisle Aas
+ * Copyright 1999-2004, Gisle Aas
  * Copyright 1999-2000, Michael A. Chase
  *
  * This library is free software; you can redistribute it and/or
@@ -27,6 +27,7 @@ literal_mode_elem[] =
     {5, "style", 1},
     {3, "xmp", 1},
     {9, "plaintext", 1},
+    {5, "title", 0},
     {8, "textarea", 0},
     {0, 0, 0}
 };
@@ -111,7 +112,7 @@ static void flush_pending_text(PSTATE* p_state, SV* self);
 static void
 report_event(PSTATE* p_state,
 	     event_id_t event,
-	     char *beg, char *end,
+	     char *beg, char *end, U32 utf8,
 	     token_pos_t *tokens, int num_tokens,
 	     SV* self
 	    )
@@ -124,6 +125,12 @@ report_event(PSTATE* p_state,
     char *argspec;
     char *s;
 
+#ifdef UNICODE_HTML_PARSER
+    #define CHR_DIST(a,b) (utf8 ? utf8_distance((a),(b)) : (a) - (b))
+#else
+    #define CHR_DIST(a,b) ((a) - (b))
+#endif
+
     /* capture offsets */
     STRLEN offset = p_state->offset;
     STRLEN line = p_state->line;
@@ -166,7 +173,7 @@ report_event(PSTATE* p_state,
 #endif
 
     /* update offsets */
-    p_state->offset += end - beg;
+    p_state->offset += CHR_DIST(end, beg);
     if (line) {
 	char *s = beg;
 	char *nl = NULL;
@@ -178,9 +185,9 @@ report_event(PSTATE* p_state,
 	    s++;
 	}
 	if (nl)
-	    p_state->column = end - nl - 1;
+	    p_state->column = CHR_DIST(end, nl) - 1;
 	else
-	    p_state->column += end - beg;
+	    p_state->column += CHR_DIST(end, beg);
     }
 
     if (event == E_NONE)
@@ -196,10 +203,13 @@ report_event(PSTATE* p_state,
 
 	if (event == E_START || event == E_END) {
 	    SV* tagname = p_state->tmp;
-	    U32 hash;
 
 	    assert(num_tokens >= 1);
 	    sv_setpvn(tagname, tokens[0].beg, tokens[0].end - tokens[0].beg);
+	    if (utf8)
+		SvUTF8_on(tagname);
+	    else
+		SvUTF8_off(tagname);
 	    if (!CASE_SENSITIVE(p_state))
 		sv_lower(aTHX_ tagname);
 
@@ -215,10 +225,8 @@ report_event(PSTATE* p_state,
 		goto IGNORE_EVENT;
 	    }
 
-	    PERL_HASH(hash, SvPVX(tagname), SvCUR(tagname));
-
 	    if (p_state->ignore_elements &&
-		hv_fetch_ent(p_state->ignore_elements, tagname, 0, hash))
+		hv_fetch_ent(p_state->ignore_elements, tagname, 0, 0))
 	    {
 		p_state->ignoring_element = newSVsv(tagname);
 		p_state->ignore_depth = 1;
@@ -226,12 +234,12 @@ report_event(PSTATE* p_state,
 	    }
 
 	    if (p_state->ignore_tags &&
-		hv_fetch_ent(p_state->ignore_tags, tagname, 0, hash))
+		hv_fetch_ent(p_state->ignore_tags, tagname, 0, 0))
 	    {
 		goto IGNORE_EVENT;
 	    }
 	    if (p_state->report_tags &&
-		!hv_fetch_ent(p_state->report_tags, tagname, 0, hash))
+		!hv_fetch_ent(p_state->report_tags, tagname, 0, 0))
 	    {
 		goto IGNORE_EVENT;
 	    }
@@ -272,8 +280,22 @@ report_event(PSTATE* p_state,
 	    p_state->pend_text_column = column;
 	    p_state->pend_text_is_cdata = p_state->is_cdata;
 	    sv_setpvn(p_state->pend_text, "", 0);
+	    if (!utf8)
+		SvUTF8_off(p_state->pend_text);
 	}
+#ifdef UNICODE_HTML_PARSER
+	if (utf8 && !SvUTF8(p_state->pend_text))
+	    sv_utf8_upgrade(p_state->pend_text);
+	if (utf8 || !SvUTF8(p_state->pend_text)) {
+	    sv_catpvn(p_state->pend_text, beg, end - beg);
+	}
+	else {
+	    SV *tmp = NULL;
+	    sv_catpvn_utf8_upgrade(p_state->pend_text, beg, end - beg, tmp);
+	}
+#else
 	sv_catpvn(p_state->pend_text, beg, end - beg);
+#endif
 	return;
     }
     else if (p_state->pend_text && SvOK(p_state->pend_text)) {
@@ -327,6 +349,8 @@ report_event(PSTATE* p_state,
 		for (i = 0; i < num_tokens; i++) {
 		    if (tokens[i].beg) {
 			prev_token = newSVpvn(tokens[i].beg, tokens[i].end-tokens[i].beg);
+			if (utf8)
+			    SvUTF8_on(prev_token);
 			av_push(av, prev_token);
 		    }
 		    else { /* boolean */
@@ -346,8 +370,8 @@ report_event(PSTATE* p_state,
 		av_extend(av, num_tokens*2);
 		for (i = 0; i < num_tokens; i++) {
 		    if (tokens[i].beg) {
-			av_push(av, newSViv(tokens[i].beg-beg));
-			av_push(av, newSViv(tokens[i].end-tokens[i].beg));
+			av_push(av, newSViv(CHR_DIST(tokens[i].beg, beg)));
+			av_push(av, newSViv(CHR_DIST(tokens[i].end, tokens[i].beg)));
 		    }
 		    else { /* boolean tag value */
 			av_push(av, newSViv(0));
@@ -366,6 +390,8 @@ report_event(PSTATE* p_state,
 	    if (num_tokens >= 1) {
 		arg = sv_2mortal(newSVpvn(tokens[0].beg,
 					  tokens[0].end - tokens[0].beg));
+		if (utf8)
+		    SvUTF8_on(arg);
 		if (!CASE_SENSITIVE(p_state) && argcode != ARG_TOKEN0)
 		    sv_lower(aTHX_ arg);
 		if (argcode == ARG_TAG && event != E_START) {
@@ -397,6 +423,8 @@ report_event(PSTATE* p_state,
 					    tokens[i].end-tokens[i].beg);
 		    SV* attrval;
 
+		    if (utf8)
+			SvUTF8_on(attrname);
 		    if (tokens[i+1].beg) {
 			char *beg = tokens[i+1].beg;
 			STRLEN len = tokens[i+1].end - beg;
@@ -405,6 +433,8 @@ report_event(PSTATE* p_state,
 			    beg++; len -= 2;
 			}
 			attrval = newSVpvn(beg, len);
+			if (utf8)
+			    SvUTF8_on(attrval);
 			if (!p_state->attr_encoded)
 			    decode_entities(aTHX_ attrval, p_state->entity2char);
 		    }
@@ -419,10 +449,8 @@ report_event(PSTATE* p_state,
 			sv_lower(aTHX_ attrname);
 
 		    if (argcode == ARG_ATTR) {
-			U32 hash;
-			PERL_HASH(hash, SvPVX(attrname), SvCUR(attrname));
-			if (hv_exists_ent(hv, attrname, hash) ||
-			    !hv_store_ent(hv, attrname, attrval, hash)) {
+			if (hv_exists_ent(hv, attrname, 0) ||
+			    !hv_store_ent(hv, attrname, attrval, 0)) {
 			    SvREFCNT_dec(attrval);
 			}
 			SvREFCNT_dec(attrname);
@@ -451,6 +479,8 @@ report_event(PSTATE* p_state,
 		for (i = 1; i < num_tokens; i += 2) {
 		    SV* attrname = newSVpvn(tokens[i].beg,
 					    tokens[i].end-tokens[i].beg);
+		    if (utf8)
+			SvUTF8_on(attrname);
 		    if (!CASE_SENSITIVE(p_state))
 			sv_lower(aTHX_ attrname);
 		    av_push(av, attrname);
@@ -461,11 +491,15 @@ report_event(PSTATE* p_state,
 	
 	case ARG_TEXT:
 	    arg = sv_2mortal(newSVpvn(beg, end - beg));
+	    if (utf8)
+		SvUTF8_on(arg);
 	    break;
 
 	case ARG_DTEXT:
 	    if (event == E_TEXT) {
 		arg = sv_2mortal(newSVpvn(beg, end - beg));
+		if (utf8)
+		    SvUTF8_on(arg);
 		if (!p_state->is_cdata)
 		    decode_entities(aTHX_ arg, p_state->entity2char);
 	    }
@@ -487,11 +521,11 @@ report_event(PSTATE* p_state,
 	    break;
 
 	case ARG_OFFSET_END:
-	    arg = sv_2mortal(newSViv(offset + (end - beg)));
+	    arg = sv_2mortal(newSViv(offset + CHR_DIST(end, beg)));
 	    break;
 
 	case ARG_LENGTH:
-	    arg = sv_2mortal(newSViv(end - beg));
+	    arg = sv_2mortal(newSViv(CHR_DIST(end, beg)));
 	    break;
 
 	case ARG_LINE:
@@ -511,6 +545,8 @@ report_event(PSTATE* p_state,
 	{
 	    int len = (unsigned char)s[1];
 	    arg = sv_2mortal(newSVpvn(s+2, len));
+	    if (SvUTF8(h->argspec))
+		SvUTF8_on(arg);
 	    s += len + 1;
 	}
 	break;
@@ -570,8 +606,21 @@ IGNORE_EVENT:
     if (p_state->skipped_text) {
 	if (event != E_TEXT && p_state->pend_text && SvOK(p_state->pend_text))
 	    flush_pending_text(p_state, self);
-	sv_catpvn(p_state->skipped_text, beg, end - beg);
+#ifdef UNICODE_HTML_PARSER
+	if (utf8 && !SvUTF8(p_state->skipped_text))
+	    sv_utf8_upgrade(p_state->skipped_text);
+	if (utf8 || !SvUTF8(p_state->skipped_text)) {
+#endif
+	    sv_catpvn(p_state->skipped_text, beg, end - beg);
+#ifdef UNICODE_HTML_PARSER
+	}
+	else {
+	    SV *tmp = NULL;
+	    sv_catpvn_utf8_upgrade(p_state->skipped_text, beg, end - beg, tmp);
+	}
+#endif
     }
+#undef CHR_DIST    
     return;
 }
 
@@ -585,6 +634,9 @@ argspec_compile(SV* src, PSTATE* p_state)
     char *s = SvPV(src, len);
     char *end = s + len;
 
+    if (SvUTF8(src))
+	SvUTF8_on(argspec);
+
     while (isHSPACE(*s))
 	s++;
 
@@ -710,8 +762,8 @@ flush_pending_text(PSTATE* p_state, SV* self)
     p_state->column        = p_state->pend_text_column;
 
     report_event(p_state, E_TEXT,
-		 SvPVX(old_pend_text), SvEND(old_pend_text),
-		 0, 0, self);
+		 SvPVX(old_pend_text), SvEND(old_pend_text), 
+		 SvUTF8(old_pend_text), 0, 0, self);
     SvOK_off(old_pend_text);
 
     p_state->unbroken_text = old_unbroken_text;
@@ -746,7 +798,7 @@ skip_until_gt(char *beg, char *end)
 }
 
 static char*
-parse_comment(PSTATE* p_state, char *beg, char *end, SV* self)
+parse_comment(PSTATE* p_state, char *beg, char *end, U32 utf8, SV* self)
 {
     char *s = beg;
 
@@ -773,7 +825,7 @@ parse_comment(PSTATE* p_state, char *beg, char *end, SV* self)
 
 		/* we are done recognizing all comments, make callbacks */
 		report_event(p_state, E_COMMENT,
-			     beg - 4, s,
+			     beg - 4, s, utf8,
 			     tokens, num_tokens,
 			     self);
 		FREE_TOKENS;
@@ -810,7 +862,7 @@ parse_comment(PSTATE* p_state, char *beg, char *end, SV* self)
 	token.end = s;
 	if (s < end) {
 	    s++;
-	    report_event(p_state, E_COMMENT, beg-4, s, &token, 1, self);
+	    report_event(p_state, E_COMMENT, beg-4, s, utf8, &token, 1, self);
 	    return s;
 	}
 	else {
@@ -834,7 +886,7 @@ parse_comment(PSTATE* p_state, char *beg, char *end, SV* self)
 		if (*s == '>') {
 		    s++;
 		    /* yup */
-		    report_event(p_state, E_COMMENT, beg-4, s, &token, 1, self);
+		    report_event(p_state, E_COMMENT, beg-4, s, utf8, &token, 1, self);
 		    return s;
 		}
 	    }
@@ -902,7 +954,7 @@ marked_section_update(PSTATE* p_state)
 
 
 static char*
-parse_marked_section(PSTATE* p_state, char *beg, char *end, SV* self)
+parse_marked_section(PSTATE* p_state, char *beg, char *end, U32 utf8, SV* self)
 {
     dTHX;
     char *s = beg;
@@ -917,6 +969,7 @@ FIND_NAMES:
     while (isHNAME_FIRST(*s)) {
 	char *name_start = s;
 	char *name_end;
+	SV *name;
 	s++;
 	while (isHNAME_CHAR(*s))
 	    s++;
@@ -928,8 +981,10 @@ FIND_NAMES:
 
 	if (!tokens)
 	    tokens = newAV();
-	av_push(tokens, sv_lower(aTHX_ newSVpvn(name_start,
-						name_end - name_start)));
+	name = newSVpvn(name_start, name_end - name_start);
+	if (utf8)
+	    SvUTF8_on(name);
+	av_push(tokens, sv_lower(aTHX_ name));
     }
     if (*s == '-') {
 	s++;
@@ -967,7 +1022,7 @@ FIND_NAMES:
 	    p_state->ms_stack = newAV();
 	av_push(p_state->ms_stack, newRV_noinc((SV*)tokens));
 	marked_section_update(p_state);
-	report_event(p_state, E_NONE, beg, s, 0, 0, self);
+	report_event(p_state, E_NONE, beg, s, utf8, 0, 0, self);
 	return s;
     }
 
@@ -983,7 +1038,7 @@ PREMATURE:
 
 
 static char*
-parse_decl(PSTATE* p_state, char *beg, char *end, SV* self)
+parse_decl(PSTATE* p_state, char *beg, char *end, U32 utf8, SV* self)
 {
     char *s = beg + 2;
 
@@ -1001,7 +1056,7 @@ parse_decl(PSTATE* p_state, char *beg, char *end, SV* self)
 	/* yes, two dashes seen */
 	s++;
 
-	tmp = parse_comment(p_state, s, end, self);
+	tmp = parse_comment(p_state, s, end, utf8, self);
 	return (tmp == s) ? beg : tmp;
     }
 
@@ -1010,7 +1065,7 @@ parse_decl(PSTATE* p_state, char *beg, char *end, SV* self)
 	/* marked section */
 	char *tmp;
 	s++;
-	tmp = parse_marked_section(p_state, s, end, self);
+	tmp = parse_marked_section(p_state, s, end, utf8, self);
 	if (!tmp)
 	    goto DECL_FAIL;
 	return (tmp == s) ? beg : tmp;
@@ -1023,7 +1078,7 @@ parse_decl(PSTATE* p_state, char *beg, char *end, SV* self)
 	token.beg = s;
 	token.end = s;
 	s++;
-	report_event(p_state, E_COMMENT, beg, s, &token, 1, self);
+	report_event(p_state, E_COMMENT, beg, s, utf8, &token, 1, self);
 	return s;
     }
 
@@ -1114,7 +1169,7 @@ parse_decl(PSTATE* p_state, char *beg, char *end, SV* self)
 	    goto PREMATURE;
 	if (*s == '>') {
 	    s++;
-	    report_event(p_state, E_DECLARATION, beg, s, tokens, num_tokens, self);
+	    report_event(p_state, E_DECLARATION, beg, s, utf8, tokens, num_tokens, self);
 	    FREE_TOKENS;
 	    return s;
 	}
@@ -1140,7 +1195,7 @@ DECL_FAIL:
 	token.beg = beg + 2;
 	token.end = s;
 	s++;
-	report_event(p_state, E_COMMENT, beg, s, &token, 1, self);
+	report_event(p_state, E_COMMENT, beg, s, utf8, &token, 1, self);
 	return s;
     }
     else {
@@ -1150,7 +1205,7 @@ DECL_FAIL:
 
 
 static char*
-parse_start(PSTATE* p_state, char *beg, char *end, SV* self)
+parse_start(PSTATE* p_state, char *beg, char *end, U32 utf8, SV* self)
 {
     char *s = beg;
     int empty_tag = 0;  /* XML feature */
@@ -1251,9 +1306,9 @@ parse_start(PSTATE* p_state, char *beg, char *end, SV* self)
     if (*s == '>') {
 	s++;
 	/* done */
-	report_event(p_state, E_START, beg, s, tokens, num_tokens, self);
+	report_event(p_state, E_START, beg, s, utf8, tokens, num_tokens, self);
 	if (empty_tag)
-	    report_event(p_state, E_END, s, s, tokens, 1, self);
+	    report_event(p_state, E_END, s, s, utf8, tokens, 1, self);
 
 	if (!p_state->xml_mode) {
 	    /* find out if this start tag should put us into literal_mode
@@ -1300,7 +1355,7 @@ PREMATURE:
 
 
 static char*
-parse_end(PSTATE* p_state, char *beg, char *end, SV* self)
+parse_end(PSTATE* p_state, char *beg, char *end, U32 utf8, SV* self)
 {
     char *s = beg+2;
     hctype_t name_first, name_char;
@@ -1332,7 +1387,7 @@ parse_end(PSTATE* p_state, char *beg, char *end, SV* self)
 	    if (*s == '>') {
 		s++;
 		/* a complete end tag has been recognized */
-		report_event(p_state, E_END, beg, s, &tagname, 1, self);
+		report_event(p_state, E_END, beg, s, utf8, &tagname, 1, self);
 		return s;
 	    }
 	}
@@ -1347,7 +1402,7 @@ parse_end(PSTATE* p_state, char *beg, char *end, SV* self)
 	    token.beg = beg + 2;
 	    token.end = s;
 	    s++;
-	    report_event(p_state, E_COMMENT, beg, s, &token, 1, self);
+	    report_event(p_state, E_COMMENT, beg, s, utf8, &token, 1, self);
 	    return s;
 	}
 	else {
@@ -1359,7 +1414,7 @@ parse_end(PSTATE* p_state, char *beg, char *end, SV* self)
 
 
 static char*
-parse_process(PSTATE* p_state, char *beg, char *end, SV* self)
+parse_process(PSTATE* p_state, char *beg, char *end, U32 utf8, SV* self)
 {
     char *s = beg + 2;  /* skip '<?' */
     /* processing instruction */
@@ -1379,7 +1434,7 @@ parse_process(PSTATE* p_state, char *beg, char *end, SV* self)
 	    }
       
 	    /* a complete processing instruction seen */
-	    report_event(p_state, E_PROCESS, beg, s,
+	    report_event(p_state, E_PROCESS, beg, s, utf8, 
 			 &token_pos, 1, self);
 	    return s;
 	}
@@ -1391,7 +1446,7 @@ parse_process(PSTATE* p_state, char *beg, char *end, SV* self)
 
 #ifdef USE_PFUNC
 static char*
-parse_null(PSTATE* p_state, char *beg, char *end, SV* self)
+parse_null(PSTATE* p_state, char *beg, char *end, U32 utf8, SV* self)
 {
     return 0;
 }
@@ -1402,7 +1457,7 @@ parse_null(PSTATE* p_state, char *beg, char *end, SV* self)
 #endif /* USE_PFUNC */
 
 static char*
-parse_buf(pTHX_ PSTATE* p_state, char *beg, char *end, SV* self)
+parse_buf(pTHX_ PSTATE* p_state, char *beg, char *end, U32 utf8, SV* self)
 {
     char *s = beg;
     char *t = beg;
@@ -1449,9 +1504,9 @@ parse_buf(pTHX_ PSTATE* p_state, char *beg, char *end, SV* self)
 		    if (*s == '>') {
 			s++;
 			if (t != end_text)
-			    report_event(p_state, E_TEXT, t, end_text,
+			    report_event(p_state, E_TEXT, t, end_text, utf8,
 					 0, 0, self);
-			report_event(p_state, E_END,  end_text, s,
+			report_event(p_state, E_END,  end_text, s, utf8,
 				     &end_token, 1, self);
 			p_state->literal_mode = 0;
 			p_state->is_cdata = 0;
@@ -1474,9 +1529,9 @@ parse_buf(pTHX_ PSTATE* p_state, char *beg, char *end, SV* self)
 			s++;
 			/* marked section end */
 			if (t != end_text)
-			    report_event(p_state, E_TEXT, t, end_text,
+			    report_event(p_state, E_TEXT, t, end_text, utf8,
 					 0, 0, self);
-			report_event(p_state, E_NONE, end_text, s, 0, 0, self);
+			report_event(p_state, E_NONE, end_text, s, utf8, 0, 0, self);
 			t = s;
 			SvREFCNT_dec(av_pop(p_state->ms_stack));
 			marked_section_update(p_state);
@@ -1501,9 +1556,9 @@ parse_buf(pTHX_ PSTATE* p_state, char *beg, char *end, SV* self)
 		    s++;
 		    if (*s == '>') {
 			s++;
-			report_event(p_state, E_TEXT, t, end_text,
+			report_event(p_state, E_TEXT, t, end_text, utf8,
 				     0, 0, self);
-			report_event(p_state, E_NONE, end_text, s,
+			report_event(p_state, E_NONE, end_text, s, utf8,
 				     0, 0, self);
 			t = s;
 			SvREFCNT_dec(av_pop(p_state->ms_stack));
@@ -1517,7 +1572,7 @@ parse_buf(pTHX_ PSTATE* p_state, char *beg, char *end, SV* self)
 	}
 	if (s != t) {
 	    if (*s == '<') {
-		report_event(p_state, E_TEXT, t, s, 0, 0, self);
+		report_event(p_state, E_TEXT, t, s, utf8, 0, 0, self);
 		t = s;
 	    }
 	    else {
@@ -1536,7 +1591,7 @@ parse_buf(pTHX_ PSTATE* p_state, char *beg, char *end, SV* self)
 		}
 		s++;
 		if (s != t)
-		    report_event(p_state, E_TEXT, t, s, 0, 0, self);
+		    report_event(p_state, E_TEXT, t, s, utf8, 0, 0, self);
 		break;
 	    }
 	}
@@ -1548,16 +1603,16 @@ parse_buf(pTHX_ PSTATE* p_state, char *beg, char *end, SV* self)
 	s++;
 
 #ifdef USE_PFUNC
-	new_pos = parsefunc[(unsigned char)*s](p_state, t, end, self);
+	new_pos = parsefunc[(unsigned char)*s](p_state, t, end, utf8, self);
 #else
 	if (isHNAME_FIRST(*s))
-	    new_pos = parse_start(p_state, t, end, self);
+	    new_pos = parse_start(p_state, t, end, utf8, self);
 	else if (*s == '/')
-	    new_pos = parse_end(p_state, t, end, self);
+	    new_pos = parse_end(p_state, t, end, utf8, self);
 	else if (*s == '!')
-	    new_pos = parse_decl(p_state, t, end, self);
+	    new_pos = parse_decl(p_state, t, end, utf8, self);
 	else if (*s == '?')
-	    new_pos = parse_process(p_state, t, end, self);
+	    new_pos = parse_process(p_state, t, end, utf8, self);
 	else
 	    new_pos = 0;
 #endif /* USE_PFUNC */
@@ -1589,6 +1644,7 @@ parse(pTHX_
       SV* self)
 {
     char *s, *beg, *end;
+    U32 utf8 = 0;
     STRLEN len;
 
     if (!chunk) {
@@ -1603,14 +1659,14 @@ parse(pTHX_
 		if (*s == '<') {
 		    /* try to parse with comments terminated with a plain '>' first */
 		    p_state->no_dash_dash_comment_end = 1;
-		    s = parse_buf(aTHX_ p_state, s, end, self);
+		    s = parse_buf(aTHX_ p_state, s, end, SvUTF8(p_state->buf), self);
 		}
 		if (*s == '<') {
 		    /* some kind of unterminated markup.  Report rest as as comment */
 		    token_pos_t token;
 		    token.beg = s + 1;
 		    token.end = end;
-		    report_event(p_state, E_COMMENT, s, end, &token, 1, self);
+		    report_event(p_state, E_COMMENT, s, end, utf8, &token, 1, self);
 		    SvREFCNT_dec(p_state->buf);
 		    p_state->buf = 0;
 		}
@@ -1621,7 +1677,7 @@ parse(pTHX_
 	    else  {
 		/* report rest as text */
 	    REST_IS_TEXT:
-		report_event(p_state, E_TEXT, s, end, 0, 0, self);
+		report_event(p_state, E_TEXT, s, end, utf8, 0, 0, self);
 		SvREFCNT_dec(p_state->buf);
 		p_state->buf = 0;
 	    }
@@ -1634,7 +1690,7 @@ parse(pTHX_
 	    SvREFCNT_dec(p_state->ignoring_element);
 	    p_state->ignoring_element = 0;
 	}
-	report_event(p_state, E_END_DOCUMENT, empty, empty, 0, 0, self);
+	report_event(p_state, E_END_DOCUMENT, empty, empty, 0, 0, 0, self);
 
 	/* reset state */
 	p_state->offset = 0;
@@ -1649,18 +1705,20 @@ parse(pTHX_
     if (p_state->buf && SvOK(p_state->buf)) {
 	sv_catsv(p_state->buf, chunk);
 	beg = SvPV(p_state->buf, len);
+	utf8 = SvUTF8(p_state->buf);
     }
     else {
 	beg = SvPV(chunk, len);
+	utf8 = SvUTF8(chunk);
 	if (p_state->offset == 0)
-	    report_event(p_state, E_START_DOCUMENT, beg, beg, 0, 0, self);
+	    report_event(p_state, E_START_DOCUMENT, beg, beg, 0, 0, 0, self);
     }
 
     if (!len)
 	return; /* nothing to do */
 
     end = beg + len;
-    s = parse_buf(aTHX_ p_state, beg, end, self);
+    s = parse_buf(aTHX_ p_state, beg, end, utf8, self);
 
     if (s == end || p_state->eof) {
 	if (p_state->buf) {
@@ -1671,13 +1729,21 @@ parse(pTHX_
 	/* need to keep rest in buffer */
 	if (p_state->buf) {
 	    /* chop off some chars at the beginning */
-	    if (SvOK(p_state->buf))
+	    if (SvOK(p_state->buf)) {
 		sv_chop(p_state->buf, s);
-	    else
+	    }
+	    else {
 		sv_setpvn(p_state->buf, s, end - s);
+		if (utf8)
+		    SvUTF8_on(p_state->buf);
+		else
+		    SvUTF8_off(p_state->buf);
+	    }
 	}
 	else {
 	    p_state->buf = newSVpv(s, end - s);
+	    if (utf8)
+		SvUTF8_on(p_state->buf);
 	}
     }
     return;
